@@ -26,7 +26,7 @@ sys_is_little_endian = (byteorder == 'little')
 import logging
 logger = logging.getLogger('pydicom')
 from dicom.datadict import DicomDictionary, dictionaryVR
-from dicom.datadict import TagForName, AllNamesForTag
+from dicom.datadict import tag_for_name, all_names_for_tag
 from dicom.tag import Tag, BaseTag
 from dicom.dataelem import DataElement, DataElement_from_raw, RawDataElement
 from dicom.valuerep import is_stringlike
@@ -62,10 +62,10 @@ class Dataset(dict):
     Example of two ways to retrieve or set values:
         
     1. dataset[0x10, 0x10].value --> patient's name
-    2. dataset.PatientsName --> patient's name
+    2. dataset.PatientName --> patient's name
 
     Example (2) is referred to as *Named tags* in this documentation.
-    PatientsName is not actually a member of the object, but unknown member
+    PatientName is not actually a member of the object, but unknown member
     requests are checked against the dicom dictionary. If the name matches a
     DicomDictionary descriptive string, the corresponding tag is used
     to look up or set the Data Element's value.
@@ -79,7 +79,7 @@ class Dataset(dict):
     def add(self, data_element):
         """Equivalent to dataset[data_element.tag] = data_element."""
         self[data_element.tag] = data_element
-    def Add(self, data_element):
+    def Add(self, data_element): # remove in v1.0
         """Deprecated -- use add()"""
         msg = ("Dataset.Add() is deprecated and will be removed in pydicom 1.0."
                " Use Dataset.add()")
@@ -91,14 +91,14 @@ class Dataset(dict):
         data_element = DataElement(tag, VR, value)
         self[data_element.tag] = data_element   # use data_element.tag since DataElement verified it
 
-    def AddNew(self, tag, VR, value):
+    def AddNew(self, tag, VR, value): #remove in v1.0
         """Deprecated -- use add_new()"""
         msg = ("Dataset.AddNew() is deprecated and will be removed in pydicom 1.0."
                " Use Dataset.add_new()")
         warnings.warn(msg, DeprecationWarning)
         self.add_new(tag, VR, value)
                 
-    def attribute(self, name):
+    def attribute(self, name): #remove in v1.0
         """Deprecated -- use Dataset.data_element()"""
         warnings.warn("Dataset.attribute() is deprecated and will be removed in pydicom 1.0. Use Dataset.data_element() instead", DeprecationWarning)
         return self.data_element(name)
@@ -109,7 +109,7 @@ class Dataset(dict):
         When using *named tags*, only the value is returned. If you want the
         whole data_element object, for example to change the data_element.VR,
         call this function with the name and the data_element instance is returned."""
-        tag = TagForName(name)
+        tag = tag_for_name(name)
         if tag:
             return self[tag]
         return None
@@ -121,7 +121,7 @@ class Dataset(dict):
 
         """
         if is_stringlike(name):
-            tag = TagForName(name)
+            tag = tag_for_name(name)
         else:
             try:
                 tag = Tag(name)
@@ -160,7 +160,7 @@ class Dataset(dict):
 
         """
         # First check if is a valid DICOM name and if we have that data element
-        tag = TagForName(name)
+        tag = tag_for_name(name)
         if tag and tag in self:
             del self[tag]
         # If not a DICOM name (or we don't have it), check for regular instance name
@@ -176,7 +176,14 @@ class Dataset(dict):
         available in an object, for example used in auto-completion in editors
         or command-line environments.
         """
-        return self.dir()
+        import inspect
+        meths = set(zip(*inspect.getmembers(Dataset,inspect.isroutine))[0])
+        props = set(zip(*inspect.getmembers(Dataset,inspect.isdatadescriptor))[0])
+        deprecated = set(('Add', 'AddNew', 'GroupDataset', 'RemovePrivateTags',
+                          'SaveAs', 'attribute', 'PixelArray'))
+        dicom_names = set(self.dir())
+        alldir=sorted((props|meths|dicom_names)-deprecated)
+        return alldir
 
     def dir(self, *filters):
         """Return a list of some or all data_element names, in alphabetical order.
@@ -191,13 +198,13 @@ class Dataset(dict):
         """
         allnames = []
         for tag, data_element in self.items():
-            allnames.extend(AllNamesForTag(tag))
+            allnames.extend(all_names_for_tag(tag))
         allnames = [x for x in allnames if x]  # remove blanks - tags without valid names (e.g. private tags)
         # Store found names in a dict, so duplicate names appear only once
         matches = {}
-        for filter in filters:
-            filter = filter.lower()
-            match = [x for x in allnames if x.lower().find(filter) != -1]
+        for filter_ in filters:
+            filter_ = filter_.lower()
+            match = [x for x in allnames if x.lower().find(filter_) != -1]
             matches.update(dict([(x,1) for x in match]))
         if filters:
             names = sorted(matches.keys())
@@ -205,7 +212,7 @@ class Dataset(dict):
         else:
             return sorted(allnames)
 
-    def file_metadata(self):
+    def file_metadata(self): # remove in v1.0
         """Return a Dataset holding only meta information (group 2).
 
         Only makes sense if this dataset is a whole file dataset.
@@ -248,7 +255,7 @@ class Dataset(dict):
         """
         # __getattr__ only called if instance cannot find name in self.__dict__
         # So, if name is not a dicom string, then is an error
-        tag = TagForName(name)
+        tag = tag_for_name(name)
         if tag is None:
             raise AttributeError, "Dataset does not have attribute '%s'." % name
         tag = Tag(tag)
@@ -268,7 +275,7 @@ class Dataset(dict):
             # If a deferred read, then go get the value now
             if data_elem.value is None:
                 from dicom.filereader import read_deferred_data_element
-                data_elem = read_deferred_data_element(self.filename, self.timestamp, data_elem)
+                data_elem = read_deferred_data_element(self.fileobj_type, self.filename, self.timestamp, data_elem)
             # Hasn't been converted from raw form read from file yet, so do so now:
             self[tag] = DataElement_from_raw(data_elem)
         return dict.__getitem__(self, tag)
@@ -284,7 +291,7 @@ class Dataset(dict):
             [(tag,data_element) for tag,data_element in self.items() if tag.group==group]
                       ))
         return ds
-    def GroupDataset(self, group):
+    def GroupDataset(self, group):  # remove in v1.0
         """Deprecated -- use group_dataset()"""
         msg = ("Dataset.GroupDataset is deprecated and will be removed in pydicom 1.0."
                " Use Dataset.group_dataset()")
@@ -334,7 +341,7 @@ class Dataset(dict):
             raise TypeError, "No pixel data found in this dataset."
 
         if not have_numpy:
-            msg = "The Numpy package is required to use PixelArray and numpy could not be imported.\n"
+            msg = "The Numpy package is required to use pixel_array, and numpy could not be imported.\n"
             raise ImportError, msg
 
         # determine the type used for the array
@@ -360,15 +367,15 @@ class Dataset(dict):
         if need_byteswap:
             arr.byteswap(True)  # True means swap in-place, don't make a new copy
         # Note the following reshape operations return a new *view* onto arr, but don't copy the data
-        if 'NumberofFrames' in self and self.NumberofFrames > 1:
-            if self.SamplesperPixel > 1:
-                arr = arr.reshape(self.SamplesperPixel, self.NumberofFrames, self.Rows, self.Columns)
+        if 'NumberOfFrames' in self and self.NumberOfFrames > 1:
+            if self.SamplesPerPixel > 1:
+                arr = arr.reshape(self.SamplesPerPixel, self.NumberOfFrames, self.Rows, self.Columns)
             else:
-                arr = arr.reshape(self.NumberofFrames, self.Rows, self.Columns)
+                arr = arr.reshape(self.NumberOfFrames, self.Rows, self.Columns)
         else:
-            if self.SamplesperPixel > 1:
+            if self.SamplesPerPixel > 1:
                 if self.BitsAllocated == 8:
-                    arr = arr.reshape(self.SamplesperPixel, self.Rows, self.Columns)
+                    arr = arr.reshape(self.SamplesPerPixel, self.Rows, self.Columns)
                 else:
                     raise NotImplementedError, "This code only handles SamplesPerPixel > 1 if Bits Allocated = 8"
             else:
@@ -401,7 +408,7 @@ class Dataset(dict):
             raise PropertyError("AttributeError in pixel_array property: " + \
                             e.args[0]), None, tb
     pixel_array = property(_get_pixel_array)
-    PixelArray = pixel_array # for backwards compatibility
+    PixelArray = pixel_array # for backwards compatibility -- remove in v1.0
 
     # Format strings spec'd according to python string formatting options
     #    See http://docs.python.org/library/stdtypes.html#string-formatting-operations
@@ -489,7 +496,7 @@ class Dataset(dict):
         Else, set an instance (python) attribute as any other class would do.
 
         """
-        tag = TagForName(name)
+        tag = tag_for_name(name)
         if tag is not None:  # successfully mapped name to a tag
             if tag not in self:  # don't have this tag yet->create the data_element instance
                 VR = dictionaryVR(tag)
@@ -519,6 +526,8 @@ class Dataset(dict):
             private_block = tag.elem >> 8
             private_creator_tag = Tag(tag.group, private_block)
             if private_creator_tag in self and tag != private_creator_tag:
+                if isinstance(data_element, RawDataElement):
+                    data_element = DataElement_from_raw(data_element)
                 data_element.private_creator = self[private_creator_tag].value
         dict.__setitem__(self, tag, data_element)
 
@@ -535,7 +544,7 @@ class Dataset(dict):
         Used in IPython, so that data element names can be found
         and offered for autocompletion on the IPython command line
         """
-        return self.dir() # does not list underlying dict properties and methods
+        return self.__dir__() # can't use dir(self) for python <2.6
 
     def update(self, dictionary):
         """Extend dict.update() to handle *named tags*."""
@@ -606,10 +615,20 @@ class FileDataset(Dataset):
         self.is_little_endian = is_little_endian
         if isinstance(filename_or_obj, basestring):
             self.filename = filename_or_obj
+            self.fileobj_type = file
         else:
-            try:
-                self.filename = filename_or_obj.name
-            except AttributeError:
+            # Note next line uses __class__ due to gzip using old-style classes 
+            #    until after python2.5 (or 2.6?)
+            # Should move to using type(filename_or_obj) when possible
+            # See http://docs.python.org/reference/datamodel.html: 
+            #   "if x is an instance of an old-style class, then x .__class__ 
+            #   designates the class of x, but type(x) is always <type 'instance'>"
+            self.fileobj_type = filename_or_obj.__class__
+            if getattr(filename_or_obj, "name", False):
+                self.filename = filename_or_obj.name  
+            elif getattr(filename_or_obj, "filename", False): #gzip python <2.7?
+                self.filename = filename_or_obj.filename
+            else:
                 self.filename = None # e.g. came from StringIO or something file-like
         self.timestamp = None
         if stat_available and self.filename and os.path.exists(self.filename):
