@@ -1,6 +1,6 @@
 # test_filereader.py
 """unittest tests for dicom.filereader module"""
-# Copyright (c) 2010 Darcy Mason
+# Copyright (c) 2010-2012 Darcy Mason
 # This file is part of pydicom, released under a modified MIT license.
 #    See the file license.txt included with this distribution, also
 #    available at http://pydicom.googlecode.com
@@ -10,6 +10,7 @@ import os
 import os.path
 import unittest
 from cStringIO import StringIO
+from decimal import Decimal
 
 import shutil
 # os.stat is only available on Unix and Windows   XXX Mac?
@@ -29,6 +30,7 @@ from dicom.filereader import read_file, data_element_generator, InvalidDicomErro
 from dicom.values import convert_value
 from dicom.tag import Tag
 from dicom.sequence import Sequence
+import gzip
 
 from warncheck import assertWarns
 
@@ -45,6 +47,8 @@ jpeg_lossless_name   = os.path.join(test_dir, "JPEG-LL.dcm")
 deflate_name = os.path.join(test_dir, "image_dfl.dcm")
 rtstruct_name = os.path.join(test_dir, "rtstruct.dcm")
 priv_SQ_name = os.path.join(test_dir, "priv_SQ.dcm")
+no_meta_group_length = os.path.join(test_dir, "no_meta_group_length.dcm")
+gzip_name = os.path.join(test_dir, "zipMR.gz")
 
 dir_name = os.path.dirname(sys.argv[0])
 save_dir = os.getcwd()
@@ -67,18 +71,19 @@ class ReaderTests(unittest.TestCase):
     def testRTPlan(self):
         """Returns correct values for sample data elements in test RT Plan file"""
         plan = read_file(rtplan_name)
-        beam = plan.Beams[0]
-        cp0, cp1 = beam.ControlPoints # if not two controlpoints, then this would raise exception
+        beam = plan.BeamSequence[0]
+        cp0, cp1 = beam.ControlPointSequence # if not two controlpoints, then this would raise exception
 
         self.assertEqual(beam.TreatmentMachineName, "unit001", "Incorrect unit name")
         self.assertEqual(beam.TreatmentMachineName, beam[0x300a, 0x00b2].value,
                 "beam TreatmentMachineName does not match the value accessed by tag number")
 
-        cumDoseRef = cp1.ReferencedDoseReferences[0].CumulativeDoseReferenceCoefficient
-        self.assert_(isClose(cumDoseRef, 0.9990268),
+        got = cp1.ReferencedDoseReferenceSequence[0].CumulativeDoseReferenceCoefficient
+        expected = Decimal('0.9990268')
+        self.assert_(got == expected,
                 "Cum Dose Ref Coeff not the expected value (CP1, Ref'd Dose Ref")
-        JawX = cp0.BLDPositions[0].LeafJawPositions
-        self.assert_(isClose(JawX[0], -100.0) and isClose(JawX[1], 100.0),
+        got = cp0.BeamLimitingDevicePositionSequence[0].LeafJawPositions
+        self.assert_(got[0] == Decimal('-100') and got[1] == Decimal('100.0'),
                 "X jaws not as expected (control point 0)")
     def testRTDose(self):
         """Returns correct values for sample data elements in test RT Dose file"""
@@ -89,8 +94,8 @@ class ReaderTests(unittest.TestCase):
                 "FrameIncrementPointer does not match the value accessed by tag number")
 
         # try a value that is nested the deepest (so deep I break it into two steps!)
-        fract = dose.ReferencedRTPlans[0].ReferencedFractionGroups[0]
-        beamnum = fract.ReferencedBeams[0].ReferencedBeamNumber
+        fract = dose.ReferencedRTPlanSequence[0].ReferencedFractionGroupSequence[0]
+        beamnum = fract.ReferencedBeamSequence[0].ReferencedBeamNumber
         self.assertEqual(beamnum, 1, "Beam number not the expected value")
     def testCT(self):
         """Returns correct values for sample data elements in test CT file...."""
@@ -101,9 +106,10 @@ class ReaderTests(unittest.TestCase):
                         ct.file_meta[0x2, 0x12].value,
                 "ImplementationClassUID does not match the value accessed by tag number")
         # (0020, 0032) Image Position (Patient)  [-158.13580300000001, -179.035797, -75.699996999999996]
-        imagepos = ct.ImagePositionPatient
-        self.assert_(isClose(imagepos, [-158.135803, -179.035797, -75.699997]),
-                "ImagePosition(Patient) values not as expected")
+        got = ct.ImagePositionPatient
+        expected = [Decimal('-158.135803'), Decimal('-179.035797'), Decimal('-75.699997')]
+        self.assert_(got == expected, "ImagePosition(Patient) values not as expected."
+                        "got %s, expected %s" % (got,expected))
         self.assertEqual(ct.Rows, 128, "Rows not 128")
         self.assertEqual(ct.Columns, 128, "Columns not 128")
         self.assertEqual(ct.BitsStored, 16, "Bits Stored not 16")
@@ -137,31 +143,54 @@ class ReaderTests(unittest.TestCase):
         got = rtss.file_meta.TransferSyntaxUID
         msg = "Expected transfer syntax %r, got %r" % (expected, got)
         self.assertEqual(expected, got, msg)
-        frame_of_ref = rtss.ReferencedFrameofReferences[0]
-        study = frame_of_ref.RTReferencedStudies[0]
-        uid = study.RTReferencedSeries[0].SeriesInstanceUID
+        frame_of_ref = rtss.ReferencedFrameOfReferenceSequence[0]
+        study = frame_of_ref.RTReferencedStudySequence[0]
+        uid = study.RTReferencedSeriesSequence[0].SeriesInstanceUID
         expected = "1.2.826.0.1.3680043.8.498.2010020400001.2.1.1"
         msg = "Expected Reference Series UID '%s', got '%s'" % (expected, uid)
         self.assertEqual(expected, uid, msg)
         
-        got = rtss.ROIContours[0].Contours[2].ContourNumber
+        got = rtss.ROIContourSequence[0].ContourSequence[2].ContourNumber
         expected = 3
         msg = "Expected Contour Number %d, got %r" % (expected, got)
         self.assertEqual(expected, got, msg)
         
-        got = rtss.RTROIObservations[0].ROIPhysicalProperties[0].ROIPhysicalProperty
+        obs_seq0 = rtss.RTROIObservationsSequence[0] 
+        got = obs_seq0.ROIPhysicalPropertiesSequence[0].ROIPhysicalProperty
         expected = 'REL_ELEC_DENSITY'
         msg = "Expected Physical Property '%s', got %r" % (expected, got)
         self.assertEqual(expected, got, msg)
-        
+    def testDir(self):
+        """Returns correct dir attributes for both Dataset and DICOM names (python >= 2.6).."""
+        # Only python >= 2.6 calls __dir__ for dir() call
+        if sys.version_info >= (2,6):
+            rtss = read_file(rtstruct_name, force=True)        
+            # sample some expected 'dir' values
+            got_dir = dir(rtss)
+            expect_in_dir = ['pixel_array', 'add_new', 'ROIContourSequence', 
+                             'StructureSetDate', '__sizeof__']
+            expect_not_in_dir = ['RemovePrivateTags', 'AddNew', 'GroupDataset'] # remove in v1.0
+            for name in expect_in_dir:
+                self.assert_(name in got_dir, "Expected name '%s' in dir()" % name)
+            for name in expect_not_in_dir:
+                self.assert_(name not in got_dir, "Unexpected name '%s' in dir()" % name)
+            # Now check for some items in dir() of a nested item
+            roi0 = rtss.ROIContourSequence[0]
+            got_dir = dir(roi0)
+            expect_in_dir = ['pixel_array', 'add_new', 'ReferencedROINumber', 
+                             'ROIDisplayColor', '__sizeof__']
+            for name in expect_in_dir:
+                self.assert_(name in got_dir, "Expected name '%s' in dir()" % name)               
     def testMR(self):
         """Returns correct values for sample data elements in test MR file....."""
         mr = read_file(mr_name)
         # (0010, 0010) Patient's Name           'CompressedSamples^MR1'
-        self.assertEqual(mr.PatientsName, 'CompressedSamples^MR1', "Wrong patient name")
-        self.assertEqual(mr.PatientsName, mr[0x10,0x10].value,
+        self.assertEqual(mr.PatientName, 'CompressedSamples^MR1', "Wrong patient name")
+        self.assertEqual(mr.PatientName, mr[0x10,0x10].value,
                 "Name does not match value found when accessed by tag number")
-        self.assert_(isClose(mr.PixelSpacing, [0.3125, 0.3125]), "Wrong pixel spacing")
+        got = mr.PixelSpacing
+        expected = [Decimal('0.3125'), Decimal('0.3125')]
+        self.assert_(got == expected, "Wrong pixel spacing")
     def testDeflate(self):
         """Returns correct values for sample data elements in test compressed (zlib deflate) file"""
         # Everything after group 2 is compressed. If we can read anything else, the decompression must have been ok.
@@ -188,6 +217,16 @@ class ReaderTests(unittest.TestCase):
         
         # Simply read the file, in 0.9.5 this generated an exception
         priv_SQ = read_file(priv_SQ_name)
+    def testNoMetaGroupLength(self):
+        """Read file with no group length in file meta..........................."""
+        # Issue 108 -- iView example file with no group length (0002,0002)
+        # Originally crashed, now check no exception, but also check one item
+        #     in file_meta, and second one in followinsg dataset
+        ds = read_file(no_meta_group_length)
+        got = ds.InstanceCreationDate
+        expected = "20111130"
+        self.assertEqual(got, expected, "Sample data element after file meta with no group length failed, expected '%s', got '%s'" % (expected, got))
+        
 
 class JPEG2000Tests(unittest.TestCase):
     def setUp(self):
@@ -198,7 +237,7 @@ class JPEG2000Tests(unittest.TestCase):
         got = self.jpeg.FrameIncrementPointer
         self.assertEqual(got, expected, "JPEG2000 file, Frame Increment Pointer: expected %s, got %s" % (expected, got))
 
-        got = self.jpeg.DerivationCodes[0].CodeMeaning
+        got = self.jpeg.DerivationCodeSequence[0].CodeMeaning
         expected = 'Lossy Compression'
         self.assertEqual(got, expected, "JPEG200 file, Code Meaning got %s, expected %s" % (got, expected))
     def testJPEG2000PixelArray(self):
@@ -210,7 +249,7 @@ class JPEGlossyTests(unittest.TestCase):
         self.jpeg = read_file(jpeg_lossy_name)
     def testJPEGlossy(self):
         """JPEG-lossy: Returns correct values for sample data elements.........."""
-        got = self.jpeg.DerivationCodes[0].CodeMeaning
+        got = self.jpeg.DerivationCodeSequence[0].CodeMeaning
         expected = 'Lossy Compression'
         self.assertEqual(got, expected, "JPEG-lossy file, Code Meaning got %s, expected %s" % (got, expected))
     def testJPEGlossyPixelArray(self):
@@ -222,7 +261,7 @@ class JPEGlosslessTests(unittest.TestCase):
         self.jpeg = read_file(jpeg_lossless_name)
     def testJPEGlossless(self):
         """JPEGlossless: Returns correct values for sample data elements........"""
-        got = self.jpeg.SourceImages[0].PurposeofReferenceCodes[0].CodeMeaning
+        got = self.jpeg.SourceImageSequence[0].PurposeOfReferenceCodeSequence[0].CodeMeaning
         expected = 'Uncompressed predecessor'
         self.assertEqual(got, expected, "JPEG-lossless file, Code Meaning got %s, expected %s" % (got, expected))
     def testJPEGlosslessPixelArray(self):
@@ -263,6 +302,14 @@ class DeferredReadTests(unittest.TestCase):
         for data_elem in ds_norm:
             tag = data_elem.tag
             self.assertEqual(data_elem.value, ds_defer[tag].value, "Mismatched value for tag %r" % tag)
+    def testZippedDeferred(self):
+        """Deferred values from a gzipped file works.............."""
+        # Arose from issue 103 "Error for defer_size read of gzip file object"
+        fobj = gzip.open(gzip_name)
+        ds = read_file(fobj, defer_size=1)
+        # before the fix, this threw an error as file reading was not in right place,
+        #    it was re-opened as a normal file, not zip file
+        num = ds.InstanceNumber
         
     def tearDown(self):
         if os.path.exists(self.testfile_name):
@@ -275,16 +322,18 @@ class FileLikeTests(unittest.TestCase):
         f = open(ct_name, 'rb')
         ct = read_file(f)
         # Tests here simply repeat testCT -- perhaps should collapse the code together?
-
+        got = ct.ImagePositionPatient
+        expected = [Decimal('-158.135803'), Decimal('-179.035797'), Decimal('-75.699997')]
+        self.assert_(got == expected, "ImagePosition(Patient) values not as expected")
         self.assertEqual(ct.file_meta.ImplementationClassUID, '1.3.6.1.4.1.5962.2',
                 "ImplementationClassUID not the expected value")
         self.assertEqual(ct.file_meta.ImplementationClassUID,
                         ct.file_meta[0x2, 0x12].value,
                 "ImplementationClassUID does not match the value accessed by tag number")
         # (0020, 0032) Image Position (Patient)  [-158.13580300000001, -179.035797, -75.699996999999996]
-        imagepos = ct.ImagePositionPatient
-        self.assert_(isClose(imagepos, [-158.135803, -179.035797, -75.699997]),
-                "ImagePosition(Patient) values not as expected")
+        got = ct.ImagePositionPatient
+        expected = [Decimal('-158.135803'), Decimal('-179.035797'), Decimal('-75.699997')]
+        self.assert_(got == expected, "ImagePosition(Patient) values not as expected")
         self.assertEqual(ct.Rows, 128, "Rows not 128")
         self.assertEqual(ct.Columns, 128, "Columns not 128")
         self.assertEqual(ct.BitsStored, 16, "Bits Stored not 16")
@@ -296,9 +345,9 @@ class FileLikeTests(unittest.TestCase):
         file_like = StringIO(open(ct_name, 'rb').read())
         ct = read_file(file_like)
         # Tests here simply repeat some of testCT test
-        imagepos = ct.ImagePositionPatient
-        self.assert_(isClose(imagepos, [-158.135803, -179.035797, -75.699997]),
-                "ImagePosition(Patient) values not as expected")
+        got = ct.ImagePositionPatient
+        expected = [Decimal('-158.135803'), Decimal('-179.035797'), Decimal('-75.699997')]
+        self.assert_(got == expected, "ImagePosition(Patient) values not as expected")        
         self.assertEqual(len(ct.PixelData), 128*128*2, "Pixel data not expected length")
         # Should also be able to close the file ourselves without exception raised:
         file_like.close()
